package cz.muni.fi.pa165.core;

import cz.muni.fi.pa165.core.device.Device;
import cz.muni.fi.pa165.core.device.DeviceService;
import cz.muni.fi.pa165.core.house.House;
import cz.muni.fi.pa165.core.house.HouseService;
import cz.muni.fi.pa165.core.metrics.Metrics;
import cz.muni.fi.pa165.core.metrics.MetricsService;
import cz.muni.fi.pa165.core.smartmeter.SmartMeter;
import cz.muni.fi.pa165.core.smartmeter.SmartMeterService;
import cz.muni.fi.pa165.core.user.User;
import cz.muni.fi.pa165.core.user.UserService;
import cz.muni.fi.pa165.core.user.UserType;
import cz.muni.fi.pa165.core.user.roles.HouseRole;
import cz.muni.fi.pa165.core.user.roles.RoleService;
import cz.muni.fi.pa165.model.dto.role.enums.HouseRoleEnum;
import cz.muni.fi.pa165.model.dto.role.enums.RoleTypeEnum;
import lombok.RequiredArgsConstructor;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

@RequiredArgsConstructor
@Component
public class DataInitializer implements ApplicationRunner {
  private final UserService userService;
  private final DeviceService deviceService;
  private final SmartMeterService smartMeterService;
  private final MetricsService metricsService;
  private final HouseService houseService;
  private final RoleService roleService;

  @Override
  public void run(ApplicationArguments args) throws Exception {
    //SeedUsers();
    //SeedDevice();
    // SeedSmartMeter();
    // SeedMetrics();
    //SeedOwnerRole();
  }

  private void SeedUsers() {
    User user =
        User.builder()
            .email("test@email.com")
            .firstName("John")
            .lastName("Doe")
            .username("johnD")
            .password("password")
            .userType(UserType.ADMIN)
            .build();
    userService.create(user);
  }

  private void SeedDevice() {
    Device device = Device.builder().name("device01").build();
    deviceService.create(device);
  }

  private void SeedSmartMeter() {
    Device device = Device.builder().name("device02").build();
    deviceService.create(device);

    SmartMeter smartMeter = SmartMeter.builder().device(device).build();
    smartMeterService.create(smartMeter);
  }

  private void SeedMetrics() {
    User user =
            User.builder()
                    .email("test@gmail.com")
                    .firstName("lol")
                    .lastName("lol")
                    .username("lol")
                    .password("lol")
                    .userType(UserType.ADMIN)
                    .build();
    userService.create(user);

    HouseRole role = HouseRole.builder()
                    .houseRole(HouseRoleEnum.Owner)
                    .build();
    role.setUser(user);
    roleService.create(role);

    Device device = Device.builder().name("device03").build();
    deviceService.create(device);

    SmartMeter smartMeter = SmartMeter.builder().device(device).build();
    smartMeterService.create(smartMeter);

    List<SmartMeter> smartMeterList = new ArrayList<SmartMeter>();
    smartMeterList.add(smartMeter);
    ArrayList<HouseRole> owners = new ArrayList<>();
    owners.add(role);
    House house = House.builder()
            .smartMeterList(smartMeterList)
            .address("lol")
            .city("lol")
            .state("lol")
            .zipcode("lol")
            .ownerList(owners)
            .build();
    houseService.create(house);



    Metrics metrics = Metrics.builder()
            .consumptionKWH(10)
            .timeStamp(LocalDateTime.now())
            .smartMeter(smartMeter)
            .build();
    metricsService.create(metrics);


  }

  private void SeedOwnerRole()
  {
    User user =
            User.builder()
                    .email("test@email.com")
                    .firstName("John")
                    .lastName("Doe")
                    .username("johnD")
                    .password("password")
                    .userType(UserType.ADMIN)
                    .build();
    user.setId("6a9f853d-59ef-4636-9c0b-bbac44bfd7f4");
    userService.create(user);

    House house = House.builder()
            .address("lol")
            .city("lol")
            .state("lol")
            .zipcode("lol")
            .build();
    house.setId("6aac9569-2d34-4aba-971b-57248fc34c80");
    houseService.create(house);

    HouseRole role = HouseRole.builder()
            .house(house)
            .build();
    role.setUser(user);
    role.setRoleType(RoleTypeEnum.HouseRole);
    roleService.create(role);
  }
}
