package cz.muni.fi.pa165.core.user;

import cz.muni.fi.pa165.core.common.DomainFacade;
import cz.muni.fi.pa165.model.dto.user.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;

import static cz.muni.fi.pa165.core.user.utils.PasswordUtil.isPasswordValid;


@Component
public class UserFacade extends DomainFacade<User, UserDto, UserCreateDto, UserUpdateDto> {

	private final UserService userService; // For the "UserService" specific methods
	private final UserMapper userMapper;   // For the "UserMapper" specific methods

	@Autowired
	public UserFacade(UserService userService, UserMapper userMapper) {
		super(userService, userMapper);
		this.userService = userService;
		this.userMapper = userMapper;
	}

	public ResponseEntity<?> logout() {
		// validate JWT token...
		return ResponseEntity.ok("Logged out successfully.");
	}

	public ResponseEntity<?> login(LoginInfoDto request) {
		String username = request.getUsername();
		String password = request.getPassword();
		if (username.isBlank()) {
			ResponseEntity.badRequest().body("Username is empty");
		}
		if (!isPasswordValid(password)) {
			ResponseEntity.badRequest().body("password is not valid");
		}
		return ResponseEntity.ok("ok!");
	}

	public ResponseEntity<?> changePassword(ChangePasswordDto request, String userId){
		String oldPassword = request.getOldPassword();
		String newPassword = request.getNewPassword();
		String confirmPassword = request.getNewPasswordConfirmation();

		// Check if new password is valid

		if (!isPasswordValid(newPassword)) {
			return ResponseEntity.badRequest().body("New password is invalid.");
		}

		// Check if new password matches confirmation password
		if (!newPassword.equals(confirmPassword)) {
			return ResponseEntity.badRequest().body("New password and confirmation password do not match.");
		}

		// TODO: Check if old password matches user's current password

		// TODO: Update user's password in the database

		return ResponseEntity.ok("Password changed successfully.");
	}
}
