package cz.muni.fi.pa165.core.user;

import cz.muni.fi.pa165.core.common.DomainService;
import jakarta.persistence.EntityNotFoundException;
import lombok.Getter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;

@Service
public class UserService extends DomainService<User> {

  @Getter private final UserRepository repository;

  @Autowired
  public UserService(UserRepository repository) {
    this.repository = repository;
  }

  @Transactional(readOnly = true)
  public User update(User user, String id) {
    int result = repository.update(user, id);
    if (result != 1) {
      throw new EntityNotFoundException("User '" + id + "' not found.");
    }
    return findById(id);
  }

  @Transactional(readOnly = true)
  public Double getConsumption(String userId,
                                     String houseId,
                                     LocalDateTime startTime,
                                     LocalDateTime endTime) {
      return repository.getStatisticsData(userId, houseId, startTime, endTime);
  }

}
