package cz.muni.fi.pa165.core.user.roles;

import cz.muni.fi.pa165.model.dto.common.Result;
import cz.muni.fi.pa165.model.dto.role.CompanyRoleDto;
import cz.muni.fi.pa165.model.dto.role.HouseRoleDto;
import cz.muni.fi.pa165.model.dto.role.RoleDto;
import cz.muni.fi.pa165.model.dto.user.UserDto;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Tag(name = "role", description = "Role API")
@RestController
@RequestMapping("/api/role")
public class RoleController {

    private final  RoleService roleService;
    private final RoleFacade roleFacade;
    @Autowired
    public RoleController(RoleService roleService, RoleFacade roleFacade){
        this.roleService = roleService;
        this.roleFacade = roleFacade;
    }

    @Operation(
            summary = "Create house role",
            description = "Creates a new user roles",
            tags = {"role"})
    @ApiResponses(
            value = {
                    @ApiResponse(
                            responseCode = "201",
                            description = "Role created",
                            content = {
                                    @Content(
                                            mediaType = "application/json",
                                            schema = @Schema(implementation = UserDto.class))
                            }),
                    @ApiResponse(responseCode = "400", description = "Invalid input", content = @Content),
            })
    @PostMapping("/house-role")
    public HouseRoleDto createHouseRole(
            @Parameter(description = "Role to be created") @RequestBody HouseRoleDto roleDto) {
        return roleFacade.createHouseRole(roleDto);
    }

    @Operation(
            summary = "Create company role",
            description = "Creates a new user roles",
            tags = {"role"})
    @ApiResponses(
            value = {
                    @ApiResponse(
                            responseCode = "201",
                            description = "Role created",
                            content = {
                                    @Content(
                                            mediaType = "application/json",
                                            schema = @Schema(implementation = UserDto.class))
                            }),
                    @ApiResponse(responseCode = "400", description = "Invalid input", content = @Content),
            })
    @PostMapping("/company-role")
    public CompanyRoleDto createCompanyRole(
            @Parameter(description = "Role to be created") @RequestBody CompanyRoleDto roleDto) {
        return roleFacade.createCompanyRole(roleDto);
        //return new CompanyRole();
    }

    @Operation(
            summary = "Create general role",
            description = "Creates a new user roles",
            tags = {"role"})
    @ApiResponses(
            value = {
                    @ApiResponse(
                            responseCode = "201",
                            description = "Role created",
                            content = {
                                    @Content(
                                            mediaType = "application/json",
                                            schema = @Schema(implementation = UserDto.class))
                            }),
                    @ApiResponse(responseCode = "400", description = "Invalid input", content = @Content),
            })
    @PostMapping("/general-role")
    public RoleDto createGeneralRole(
            @Parameter(description = "Role to be created") @RequestBody RoleDto roleDto) {
        return roleFacade.createGeneralRole(roleDto);
    }


    @DeleteMapping("/{id}")
    @Operation(summary = "Delete a user role")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Role deleted successfully"),
            @ApiResponse(responseCode = "404", description = "Role not found"),
            @ApiResponse(responseCode = "500", description = "Internal server error")
    })
    public RoleDto deleteById(
            @Parameter(description = "ID of the role to be deleted") @PathVariable String id) {
        return roleFacade.deleteRole(id);
    }

    @Operation(
            summary = "Get all user roles",
            description = "Returns all user roles",
            tags = {"role"})
    @ApiResponses(
            value = {
                    @ApiResponse(
                            responseCode = "200",
                            description = "Roles found",
                            content = {
                                    @Content(
                                            mediaType = "application/json",
                                            schema = @Schema(implementation = Result.class))
                            })
            })
    @GetMapping
    public List<RoleDto> findUserRoles(
            @Parameter(description = "Page number of results to retrieve") @RequestParam String userId) {
        return roleService.findAllByUser(userId);
    }
}
