package cz.muni.fi.pa165.core.company;

import com.fasterxml.jackson.databind.ObjectMapper;
import cz.muni.fi.pa165.core.user.UserController;
import cz.muni.fi.pa165.model.dto.company.CompanyCreateDto;
import cz.muni.fi.pa165.model.dto.company.CompanyDto;
import cz.muni.fi.pa165.model.dto.user.UserCreateDto;
import cz.muni.fi.pa165.model.dto.user.UserDto;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.web.servlet.MockMvc;

import static org.assertj.core.api.AssertionsForClassTypes.assertThat;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

/**
 * Integration tests for the {@link CompanyController} class.
 */
@SpringBootTest
@AutoConfigureMockMvc
public class CompanyControllerTest {

	@Autowired
	private MockMvc mockMvc;

	@Autowired
	private ObjectMapper objectMapper;

	private final static String URL = "/api/company";
	private final static String CONTENT_TYPE =  "application/json";

	/**
	 * Tests the {@link CompanyController#create(CompanyCreateDto)} method with valid input.
	 * Expects the response status code to be 201 (Created).
	 */
	@Test
	@DisplayName("Create company with valid input")
	void createNonExistingCompanyTest() throws Exception {
		// Prepare
		CompanyCreateDto createDto = new CompanyCreateDto();
		createDto.setName("superDuperCompany");

		// Execute
		String response = mockMvc.perform(post(URL)
						.contentType(CONTENT_TYPE)
						.content(objectMapper.writeValueAsString(createDto)))
				.andExpect(status().isCreated())
				.andReturn()
				.getResponse()
				.getContentAsString();

		// Verify
		CompanyDto companyDto = objectMapper.readValue(response, CompanyDto.class);
		assertThat(companyDto.getId()).isNotNull();
		assertThat(companyDto.getName()).isEqualTo(createDto.getName());
	}
}
