package cz.muni.fi.pa165.core.company;

import cz.muni.fi.pa165.core.house.HouseRepository;
import cz.muni.fi.pa165.core.smartmeter.SmartMeterRepository;
import cz.muni.fi.pa165.core.user.UserRepository;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.boot.test.autoconfigure.orm.jpa.TestEntityManager;
import org.springframework.test.context.junit4.SpringRunner;

import java.util.ArrayList;
import java.util.regex.Pattern;

import static org.assertj.core.api.Assertions.assertThat;
@RunWith(SpringRunner.class)
@DataJpaTest
public class UnitTestCompanyJPA {
    @Autowired
    private TestEntityManager entityManager;
    @Autowired
    private CompanyRepository companyRepository;
    @Autowired
    private HouseRepository houseRepository;
    @Autowired
    private SmartMeterRepository smartMeterRepository;
    @Autowired
    private UserRepository userRepository;

    @Test
    public void createCompanyTesting() throws Exception {
        String company = this.entityManager.persistAndGetId(new Company()).toString();
        // Regular expression to match UUID format
        Pattern uuidPattern = Pattern.compile(
                "[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}",
                Pattern.CASE_INSENSITIVE);
        // Check if the string matches the UUID format
        boolean isUUID = uuidPattern.matcher(company).matches();
        assertThat(isUUID).isTrue();
    }

    @Test
    public void shouldFindNoneIfRepositoryIsEmpty() {
        Iterable<Company> companies = companyRepository.findAll();
        assertThat(companies).isEmpty();
    }

    private Company helperRegister() {
        return companyRepository.save(new Company("Tesla", new ArrayList<>()));
    }

    @Test
    public void shouldStore() {
        Company company = helperRegister();

        assertThat(company).hasFieldOrPropertyWithValue("name", "Tesla");
    }

    @Test
    public void shouldFindAll() {
        Company company = new Company();
        entityManager.persist(company);

        Company company1 = new Company();
        entityManager.persist(company1);

        Company company2 = new Company();
        entityManager.persist(company2);

        assertThat(companyRepository.findAll()).hasSize(3).contains(company, company1, company2);
    }

    @Test
    public void shouldFindById() {
        Company company = new Company();
        entityManager.persist(company);

        Company company1 = new Company();
        entityManager.persist(company1);

        Company company2 = new Company();
        entityManager.persist(company2);

        Company found = companyRepository.findById(company2.getId()).get();

        assertThat(found).isEqualTo(company2);

        found = companyRepository.findById(company1.getId()).get();

        assertThat(found).isEqualTo(company1);
    }

    @Test
    public void shouldUpdateById() {
        Company company = new Company();
        entityManager.persist(company);

        Company company1 = new Company();
        entityManager.persist(company1);

        Company updateCompany = companyRepository.findById(company1.getId()).get();
        updateCompany.setName("Gorenje");
        companyRepository.save(updateCompany);

        Company check = companyRepository.findById(company1.getId()).get();

        assertThat(check.getId()).isEqualTo(company1.getId());
        assertThat(check.getName()).isEqualTo(company1.getName());
    }

    @Test
    public void shouldDeleteById() {
        Company company = new Company();
        entityManager.persist(company);

        Company company1 = helperRegister();
        entityManager.persist(company1);

        companyRepository.deleteById(company1.getId());

        assertThat(companyRepository.findAll()).hasSize(1).contains(company);
        assertThat(companyRepository.findAll()).hasSize(1).doesNotContain(company1);
    }

    @Test
    public void shouldDeleteAll() {
        entityManager.persist(helperRegister());
        entityManager.persist(helperRegister());

        companyRepository.deleteAll();

        assertThat(companyRepository.findAll()).isEmpty();
    }
}
