package cz.muni.fi.pa165.core.device;

import cz.muni.fi.pa165.core.house.HouseRepository;
import cz.muni.fi.pa165.core.manufacturer.Manufacturer;
import cz.muni.fi.pa165.core.manufacturer.ManufacturerRepository;
import cz.muni.fi.pa165.core.smartmeter.SmartMeter;
import cz.muni.fi.pa165.core.smartmeter.SmartMeterRepository;
import cz.muni.fi.pa165.core.user.UserRepository;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.boot.test.autoconfigure.orm.jpa.TestEntityManager;
import org.springframework.test.context.junit4.SpringRunner;

import java.util.List;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

import static org.assertj.core.api.Assertions.assertThat;

@RunWith(SpringRunner.class)
@DataJpaTest
public class UnitTestDeviceJPA {
    @Autowired
    private TestEntityManager entityManager;
    @Autowired
    private HouseRepository houseRepository;
    @Autowired
    private SmartMeterRepository smartMeterRepository;
    @Autowired
    private DeviceRepository deviceRepository;
    @Autowired
    private ManufacturerRepository manufacturerRepository;
    @Autowired
    private UserRepository userRepository;

    @Test
    public void createDeviceTesting() throws Exception {
        String device = this.entityManager.persistAndGetId(new Device()).toString();
        // Regular expression to match UUID format
        Pattern uuidPattern = Pattern.compile(
                "[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}",
                Pattern.CASE_INSENSITIVE);
        // Check if the string matches the UUID format
        boolean isUUID = uuidPattern.matcher(device).matches();
        assertThat(isUUID).isTrue();
    }

    @Test
    public void shouldFindNoDevicesIfRepositoryIsEmpty() {
        Iterable<Device> devices = deviceRepository.findAll();
        assertThat(devices).isEmpty();
    }

    private Device helperRegisterDevice() {
        Iterable<SmartMeter> smartMeterIterable;
        smartMeterIterable = smartMeterRepository.saveAll(List.of(new SmartMeter(), new SmartMeter(), new SmartMeter()));
        List<SmartMeter> list = StreamSupport.stream(smartMeterIterable.spliterator(), false)
                .collect(Collectors.toList());
        //need to save a user to the database first to asign him to the new device
        Manufacturer manufacturer = manufacturerRepository.save(new Manufacturer());
        return deviceRepository.save(new Device("Tesla Smart Meter A134", list, manufacturer));
    }

    @Test
    public void shouldStoreADevice() {
        Device device = helperRegisterDevice();

        assertThat(device).hasFieldOrPropertyWithValue("name", "Tesla Smart Meter A134");
    }

    @Test
    public void shouldFindAllDevices() {
        Device device1 = new Device();
        entityManager.persist(device1);

        Device device2 = new Device();
        entityManager.persist(device2);

        assertThat(deviceRepository.findAll()).hasSize(2).contains(device1, device2);
    }

    @Test
    public void shouldFindDeviceById() {
        Device device1 = new Device();
        entityManager.persist(device1);

        Device device2 = new Device();
        entityManager.persist(device2);

        Device foundDevice = deviceRepository.findById(device2.getId()).get();

        assertThat(foundDevice).isEqualTo(device2);

        foundDevice = deviceRepository.findById(device1.getId()).get();

        assertThat(foundDevice).isEqualTo(device1);
    }

    @Test
    public void shouldUpdateDeviceById() {
        Device device1 = new Device();
        entityManager.persist(device1);

        Device device2 = new Device();
        entityManager.persist(device2);



        Device updatedDevice = deviceRepository.findById(device2.getId()).get();
        updatedDevice.setName("Smart Meter Gorenje V245");
        deviceRepository.save(updatedDevice);

        Device checkDevice = deviceRepository.findById(device2.getId()).get();

        assertThat(checkDevice.getId()).isEqualTo(device2.getId());
        assertThat(checkDevice.getName()).isEqualTo(device2.getName());
        assertThat(checkDevice.getManufacturer()).isEqualTo(device2.getManufacturer());
        assertThat(checkDevice.getCreatedDateTime()).isEqualTo(device2.getCreatedDateTime());
    }

    @Test
    public void shouldDeleteById() {
        Device device1 = new Device();
        entityManager.persist(device1);

        Device device2 = new Device();
        entityManager.persist(device2);

        Device device = helperRegisterDevice();
        entityManager.persist(device);

        deviceRepository.deleteById(device.getId());

        assertThat(deviceRepository.findAll()).hasSize(2).doesNotContain(device);
    }

    @Test
    public void shouldDeleteAll() {
        entityManager.persist(helperRegisterDevice());
        entityManager.persist(helperRegisterDevice());

        houseRepository.deleteAll();

        assertThat(houseRepository.findAll()).isEmpty();
    }
}