package cz.muni.fi.pa165.core.manufacturer;

import cz.muni.fi.pa165.core.device.Device;
import cz.muni.fi.pa165.core.device.DeviceRepository;
import cz.muni.fi.pa165.core.house.HouseRepository;
import cz.muni.fi.pa165.core.smartmeter.SmartMeterRepository;
import cz.muni.fi.pa165.core.user.UserRepository;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.boot.test.autoconfigure.orm.jpa.TestEntityManager;
import org.springframework.test.context.junit4.SpringRunner;

import java.util.List;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

import static org.assertj.core.api.Assertions.assertThat;

@RunWith(SpringRunner.class)
@DataJpaTest
public class UnitTestManufacturerJPA {
    @Autowired
    private TestEntityManager entityManager;
    @Autowired
    private HouseRepository houseRepository;
    @Autowired
    private SmartMeterRepository smartMeterRepository;
    @Autowired
    private UserRepository userRepository;

    @Autowired
    private ManufacturerRepository manufacturerRepository;

    @Autowired
    private DeviceRepository deviceRepository;

    @Test
    public void createTesting() throws Exception {
        String manufacturer = this.entityManager.persistAndGetId(new Manufacturer()).toString();
        // Regular expression to match UUID format
        Pattern uuidPattern = Pattern.compile(
                "[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}",
                Pattern.CASE_INSENSITIVE);
        // Check if the string matches the UUID format
        boolean isUUID = uuidPattern.matcher(manufacturer).matches();
        assertThat(isUUID).isTrue();
    }

    @Test
    public void shouldFindNoneIfRepositoryIsEmpty() {
        Iterable<Manufacturer> manufacturers = manufacturerRepository.findAll();
        assertThat(manufacturers).isEmpty();
    }

    private Manufacturer helperRegister() {
        Iterable<Device> deviceIterable;
        deviceIterable = deviceRepository.saveAll(List.of(new Device(), new Device(), new Device()));
        List<Device> list = StreamSupport.stream(deviceIterable.spliterator(), false)
                .collect(Collectors.toList());
        return manufacturerRepository.save(new Manufacturer("Manufacturer1", list));
    }

    @Test
    public void shouldStore() {
        Manufacturer manufacturer = helperRegister();

        assertThat(manufacturer).hasFieldOrPropertyWithValue("name", "Manufacturer1");
    }

    @Test
    public void shouldFindAll() {
        Manufacturer m = new Manufacturer();
        entityManager.persist(m);

        Manufacturer m1 = new Manufacturer();
        entityManager.persist(m1);

        Manufacturer m2 = new Manufacturer();
        entityManager.persist(m2);

        assertThat(manufacturerRepository.findAll()).hasSize(3).contains(m, m1, m2);
    }

    @Test
    public void shouldFindById() {
        Manufacturer m = new Manufacturer();
        entityManager.persist(m);

        Manufacturer m1 = new Manufacturer();
        entityManager.persist(m1);

        Manufacturer m2 = new Manufacturer();
        entityManager.persist(m2);

        Manufacturer found = manufacturerRepository.findById(m.getId()).get();

        assertThat(found).isEqualTo(m);

        found = manufacturerRepository.findById(m.getId()).get();

        assertThat(found).isEqualTo(m);
    }

    @Test
    public void shouldUpdateById() {
        Manufacturer m = new Manufacturer();
        entityManager.persist(m);

        Manufacturer m1 = new Manufacturer();
        entityManager.persist(m1);

        Manufacturer update = manufacturerRepository.findById(m.getId()).get();
        update.setName("New name");
        manufacturerRepository.save(update);

        Manufacturer check = manufacturerRepository.findById(m.getId()).get();

        assertThat(check.getId()).isEqualTo(m.getId());
        assertThat(check.getName()).isEqualTo(m.getName());
    }

    @Test
    public void shouldDeleteById() {
        Manufacturer m = new Manufacturer();
        entityManager.persist(m);

        Manufacturer m1 = new Manufacturer();
        entityManager.persist(m1);

        manufacturerRepository.deleteById(m.getId());

        assertThat(manufacturerRepository.findAll()).hasSize(1).contains(m1);
        assertThat(manufacturerRepository.findAll()).hasSize(1).doesNotContain(m);
    }

    @Test
    public void shouldDeleteAll() {
        entityManager.persist(helperRegister());
        entityManager.persist(helperRegister());

        manufacturerRepository.deleteAll();

        assertThat(manufacturerRepository.findAll()).isEmpty();
    }
}

