package cz.muni.fi.pa165.core.user;

import com.fasterxml.jackson.databind.ObjectMapper;
import cz.muni.fi.pa165.model.dto.user.UserCreateDto;
import cz.muni.fi.pa165.model.dto.user.UserDto;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.web.servlet.MockMvc;

import static org.assertj.core.api.AssertionsForClassTypes.assertThat;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

/**
 * Integration tests for the {@link UserController} class.
 */
@SpringBootTest
@AutoConfigureMockMvc
public class UserControllerTest {

	@Autowired
	private MockMvc mockMvc;

	@Autowired
	private ObjectMapper objectMapper;

	private final static String URL = "/api/user";
	private final static String CONTENT_TYPE =  "application/json";

	/**
	 * Tests the {@link UserController#create(UserCreateDto)} method with valid input.
	 * Expects the response status code to be 201 (Created).
	 */
	@Test
	@DisplayName("Create user with valid input")
	void createNonExistingUserTest() throws Exception {
		// Prepare
		UserCreateDto createDto = new UserCreateDto();
		createDto.setUsername("testUser");
		createDto.setPassword("testPassword");
		createDto.setFirstName("John");
		createDto.setLastName("Doe");
		createDto.setEmail("johndoe@test.com");

		// Execute
		String response = mockMvc.perform(post(URL)
						.contentType(CONTENT_TYPE)
						.content(objectMapper.writeValueAsString(createDto)))
				.andExpect(status().isCreated())
				.andReturn()
				.getResponse()
				.getContentAsString();

		// Verify
		UserDto userDto = objectMapper.readValue(response, UserDto.class);
		assertThat(userDto.getId()).isNotNull();
		assertThat(userDto.getUsername()).isEqualTo(createDto.getUsername());
		assertThat(userDto.getEmail()).isEqualTo(createDto.getEmail());
		assertThat(userDto.getFirstName()).isEqualTo(createDto.getFirstName());
		assertThat(userDto.getLastName()).isEqualTo(createDto.getLastName());
	}
}
