package cz.muni.fi.pa165.microservice3.email;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import com.fasterxml.jackson.databind.ObjectMapper;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@AutoConfigureMockMvc
class EmailControllerTest {

	@Autowired
	private MockMvc mockMvc;

	// injected mapper for converting classes into JSON strings
	@Autowired private ObjectMapper objectMapper;

	@Test
	void badRequestWithInvalidAddress() throws Exception {
		EmailTemplate emailTemplate = new EmailTemplate();
		emailTemplate.setSendTo("email");
		emailTemplate.setSubject("Test");
		emailTemplate.setBody("Hello");
		mockMvc
				.perform(post("/api/email/sendemail")
						.contentType(MediaType.APPLICATION_JSON)
						.content(objectMapper.writeValueAsString(emailTemplate)))
				.andExpect(status().isBadRequest());
	}

	@Test
	void sendEmailToValidAddress() throws Exception {
		EmailTemplate emailTemplate = new EmailTemplate();
		emailTemplate.setSendTo("email@gmail.com");
		emailTemplate.setSubject("Test");
		emailTemplate.setBody("Hello");
		mockMvc
				.perform(post("/api/email/sendemail")
						.contentType(MediaType.APPLICATION_JSON)
						.content(objectMapper.writeValueAsString(emailTemplate)))
				.andExpect(status().isOk());
	}

}