package cz.muni.fi.pa165.microservice4;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;

import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalTime;

import static org.hamcrest.Matchers.containsString;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
@SpringBootTest
@AutoConfigureMockMvc
class ElectricityPriceControllerTest {

    @Autowired
    private MockMvc mockMvc;
    //dummy data from the list in service class (before using database)
    private static final String data = "{\"id\":\"1\",\"companyId\":\"1\",\"priceHighTariff\":300.0,\"priceLowTariff\":200.0}";

    @Test
    public void shouldReturnEntranceData() throws Exception {
        this.mockMvc.perform(get("/api/v1/electricity/electricityprices"))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().string(containsString(data)));
    }
    @Test
    void testGetAllElectricityPrice() throws Exception {
        mockMvc.perform(get("/api/v1/electricity/electricityprices")
                .contentType(MediaType.APPLICATION_JSON_VALUE)
                .accept(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(status().isOk());
    }

    @Test
    public void shouldRegisterElectricityPrice() throws Exception {
        this.mockMvc.perform( MockMvcRequestBuilders
                        .post("/api/v1/electricity/electricityprices")
                        .content("{\"companyId\":\"2\",\"priceHighTariff\":700.0,\"priceLowTariff\":400.0}")
                        .contentType(MediaType.APPLICATION_JSON)
                        .accept(MediaType.APPLICATION_JSON))
                .andExpect(MockMvcResultMatchers.jsonPath("$.id").exists());
    }

    @Test
    public void shouldNotThrowElectricityPriceForCompanyException() throws Exception {
        this.mockMvc.perform(delete("/api/v1/electricity/electricityprices/2"))
                .andDo(print())
                .andExpect(status().is(200))
                .andExpect(content().string(containsString("deleted electricity prices for company with companyId " + 2)));
    }
    @Test
    public void shouldReturnPriceAccordingToTime() throws Exception {
        if(LocalDate.now().getDayOfWeek() == DayOfWeek.SATURDAY || LocalDate.now().getDayOfWeek() == DayOfWeek.SUNDAY)
            this.mockMvc.perform(get("/api/v1/electricity/electricityprices/1"))
                    .andDo(print())
                    .andExpect(status().isOk())
                    .andExpect(content().string(containsString("200.0")));
        else if(LocalTime.now().getHour() > 7 && LocalTime.now().getHour() < 21)
            this.mockMvc.perform(get("/api/v1/electricity/electricityprices/1"))
                    .andDo(print())
                    .andExpect(status().isOk())
                    .andExpect(content().string(containsString("300.0")));
        else
            this.mockMvc.perform(get("/api/v1/electricity/electricityprices/1"))
                    .andDo(print())
                    .andExpect(status().isOk())
                    .andExpect(content().string(containsString("200.0")));
    }

    @Test
    public void shouldThrowElectricityPriceForCompanyException() throws Exception {
        this.mockMvc.perform(delete("/api/v1/electricity/electricityprices/66"))
                .andExpect(status().is(400));

    }

    @Test
    public void shouldThrowExceptionForNoElectricityCompany() throws Exception {
        this.mockMvc.perform(get("/api/v1/electricity/electricityprices/notanid"))
                .andExpect(status().is(400));
    }
}
